#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "collection.h"

static void CheckBound( Collection *mq, int i )
{ 
	if( i >= mq->dataSize || i < 0 )
		printf( "trap %i--%i \n", i, mq->dataSize );
	assert( mq );
	assert( i < mq->dataSize || i >= 0 );
//	{
//		fprintf( stderr, " current array index is %d/%d/%d \n", 
//					i, mq->dataSize, mq->maxDataSize );
//		ReExit( -1 );
//	}
}

static int AddMemory( Collection *mq)
{
	unsigned long *newPtr;
	int i;

	mq->maxDataSize += mq->delta;
	newPtr  = (unsigned long*) malloc(mq->maxDataSize * sizeof( unsigned long ));
	for( i=0; i<mq->dataSize; i++ )
		newPtr[i] = mq->dataPtr[i];

	free( mq->dataPtr );
	mq->dataPtr = (unsigned long*) malloc( sizeof( unsigned long ) * mq->maxDataSize);

	for(i=0; i<mq->dataSize; i++ )
		mq->dataPtr[i] = newPtr[i];
	free( newPtr );

	return !!(mq->dataSize+1);
}

static int ReAllocateData( Collection *mq, int i )
{
	int ii;
	mq->dataSize--;
	for( ii=i; ii<mq->dataSize; ii++ )
		mq->dataPtr[ii] = mq->dataPtr[ii+1];

	return mq->dataSize;
}

int InitCollection( Collection *mq, int _dataSize, int _delta )
{
	if( _dataSize < 0 )
		_dataSize = 0;
	mq->maxDataSize = _dataSize;

	mq->delta = _delta;
	mq->dataSize = 0;

	if( mq->maxDataSize > 0 )
		mq->dataPtr = (unsigned long*) malloc(mq->maxDataSize*sizeof(unsigned long ));

	return mq->dataSize;
}

int AddItem( Collection *mq, void *ptr )
{
	assert( ptr );

	if( mq->dataSize >= mq->maxDataSize )
		AddMemory(mq);

	mq->dataPtr[mq->dataSize++] = (unsigned long)ptr;
	
	return mq->dataSize-1;
}


int InsertItem( Collection *mq, int i, void *ptr )
{
	assert( ptr );

	CheckBound( mq, i );
	if( i < mq->dataSize )
	{
		mq->dataPtr[i] = (unsigned long)ptr;
		return 1;
	}

	return -1;
}

int RemoveItembyAddress( Collection *mq, void *ptr )
{
	int i;
	for( i=0; i< mq->dataSize; i++ )
	{
		if( ptr == (void*)mq->dataPtr[i] )
		{
			free( (void*) mq->dataPtr[i]);
			return ReAllocateData( mq, i );
		}
	}

	return 0;
}

int RemoveItembyIndex( Collection *mq, int i )
{
	CheckBound( mq, i );
	free( (void*) mq->dataPtr[i] );
	return ReAllocateData( mq, i );
}


int RemoveAllItems( Collection *mq )
{
	int i;
	for(i=0; i< mq->dataSize; i++ )
		free( (void*) mq->dataPtr[i] );
	free( mq->dataPtr );
	mq->dataPtr = NULL;
	mq->dataSize = mq->maxDataSize = 0;

	return mq->dataSize;
}

int DeleteItembyAddress( Collection *mq, void *ptr )
{
	int i;
	for(i=0; i< mq->dataSize; i++ )
	{
		if( ptr == (void*)mq->dataPtr[i] )
			return ReAllocateData( mq, i );
	}

	return -1;
}

int DeleteItembyIndex( Collection *mq, int i )
{
	CheckBound( mq, i );
	return ReAllocateData( mq, i );
}

int DeleteAllItems( Collection *mq)
{
	mq->dataSize = mq->maxDataSize = 0;
	free( mq->dataPtr );
	mq->dataPtr = NULL;

	return mq->dataSize;
}


void *GetItembyIndex( Collection *mq, int i)
{
	CheckBound( mq, i );
	return (void*) mq->dataPtr[i];
}

void *GetLastItem( Collection *mq )
{
	return GetItembyIndex( mq, mq->dataSize-1 );
}

void *GetFirstItem( Collection *mq )
{
	return GetItembyIndex( mq, 0 );
}

int GetIndexbyItem( Collection *mq, void *ptr )
{
	int i=0;

	for( ; i< mq->dataSize; i++ )
	{
		if( mq->dataPtr[i] == (unsigned long)ptr )
			break;
	}
	return i == mq->dataSize? -1 : i;
}

void ChangeItemsPlace( Collection *mq, int i, int j )
{	
	unsigned long s ;
	CheckBound( mq, i );
	CheckBound( mq, j );

	s = mq->dataPtr[i];
	mq->dataPtr[i] = mq->dataPtr[j];
	mq->dataPtr[j] = s;
}
