#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

#include "pglos.h"
#include "abimport.h"
#include "proto.h"

char curGlos[256] = { 0 };
char GlossaryPath[_MAX_PATH] = "";
Collection glosSet;

static Collection *CheckGlossaries()
{
	DIR *dp;
	struct dirent *dr;
	PtArg_t arg[2];
	Collection *cl = calloc( sizeof( Collection ), 1 );
	char **glos;
	int i;
	ushort_t sel = 1;

	InitCollection( cl, 5, 5 );

	dp = opendir( GlossaryPath );
	if( !dp )
	{
		free( cl );
		return NULL;
	}

	while( dr = readdir( dp ))
	{
		if( !S_ISDIR( dr->d_stat.st_mode ) && !fnmatch( "*.dwa", dr->d_name, 0 ))
			AddItem( cl, strdup( dr->d_name ));
	}
	closedir( dp );

	if( !cl->dataSize )
	{
		PtMessageBox( ABW_base, "Warning!", "Glossaries Not Found", NULL, NULL );
		free( cl );
		return NULL;
	}

	glos = (char**)calloc( cl->dataSize, sizeof( void* ));
	for( i=0; i<cl->dataSize; i++ )
	{
		glos[i] = (char*)GetItembyIndex( cl, i );
		glos[i][strlen(glos[i])-4] = '\0';
		if( !strcmp( glos[i], curGlos ))
			sel = i+1;
	}

	if( !strlen( curGlos ))
		strcpy( curGlos, glos[0] );

	PtSetArg( &arg[0], Pt_ARG_ITEMS, glos, i );
	PtSetArg( &arg[1], Pt_ARG_CBOX_SEL_ITEM, sel, 0);
	PtSetResources( ABW_gls_cmb, 2, arg);

	free( glos );
	return cl;
}

int ReadConfig()
{
	char rcfile[_MAX_PATH];
	char *cfg_str, val[128];
	Collection *cl;
	int i=0;
	GlossarySetting_t *glset;

	sprintf( rcfile, "%s/%s", getenv( "HOME" ), CONFIG_NAME );
	
	if( PxConfigOpen( rcfile, PXCONFIG_READ ) != Pt_FALSE )
	{
		if( PxConfigSection( "main" ) == Pt_TRUE )
		{
			while(( cfg_str = PxConfigNextString( val, sizeof( val ))) != NULL )
			{
				if( !strlen( GlossaryPath ) && 
					!strcmp( cfg_str, "glossary_path" ))
					strcpy( GlossaryPath, val );
				else if( !strcmp( cfg_str, "current_glossary" ))
					strcpy( curGlos, val );
			}
		}

		if( !strlen( GlossaryPath ))
			strcpy( GlossaryPath, "/usr/local/lib/glossary" );

		InitCollection( &glosSet, 10, 5 );

		if( PxConfigSection( "default" ) == Pt_TRUE )
		{
			glset = calloc( sizeof( GlossarySetting_t ), 1  );
			strcpy( glset->name, "default" );
			while(( cfg_str = PxConfigNextString( val, sizeof( val ))) != NULL )
			{
				if( !strcmp( cfg_str, "skip_lines" ))
					glset->skip_line = atol( val );
				else if( !strcmp( cfg_str, "delimiter" ))
				{
					val[strlen(val)-1] = '\0';
					strcpy( glset->delimiter, &val[1] );
				}
			}
			AddItem( &glosSet, glset );
		}

		cl = CheckGlossaries();
		if( cl )
		{
			for( ; i<cl->dataSize; i++ )
			{
				if( PxConfigSection( (char*)GetItembyIndex( cl, i )) == Pt_TRUE )
				{
					glset = calloc( sizeof( GlossarySetting_t ), 1  );
					strcpy( glset->name, (char*)GetItembyIndex( cl, i ));
					while(( cfg_str = PxConfigNextString( val, sizeof( val ))) != NULL )
					{
						if( !strcmp( cfg_str, "skip_lines" ))
							glset->skip_line = atol( val );
						else if( !strcmp( cfg_str, "delimiter" ))
						{
							val[strlen(val)-1] = '\0';
							strcpy( glset->delimiter, &val[1] );
						}
					}
					AddItem( &glosSet, glset );
				}
			}
			RemoveAllItems( cl );
			free( cl );
		}else
		{
			glset = calloc( sizeof( GlossarySetting_t ), 1  );
			strcpy( glset->name, "default" );
			glset->skip_line = 0;
			strcpy( glset->delimiter, "=" );
			AddItem( &glosSet, glset );
			glset = calloc( sizeof( GlossarySetting_t ), 1  );
			strcpy( glset->name, "muller7" );
			glset->skip_line = 213;
			strcpy( glset->delimiter, "  " );
			AddItem( &glosSet, glset );
		}
		PxConfigClose();
	}

	if( !strlen( GlossaryPath ))
	{
		strcpy( GlossaryPath, "/usr/local/lib/glossary/" );
		WriteConfig();
		PtMessageBox( ABW_base, "Warning!", "Not Found Glossary Path!\n"
			"Default Glossary Path is /usr/local/lib/glossary/", NULL, NULL );
	}
	return 0;
}

int WriteConfig()
{
	char rcfile[_MAX_PATH];
	int i=0;
	GlossarySetting_t *glset;
	char buf[256];

	sprintf( rcfile, "%s/%s", getenv( "HOME" ), CONFIG_NAME );

	if( PxConfigOpen( rcfile, PXCONFIG_WRITE | PXCONFIG_CREATE ) == Pt_FALSE )
	{
		PtMessageBox( ABW_base, "Config Error!", "Unable Write to Config", NULL, NULL );
		return 1;
	}
	
	PxConfigWriteString( "main", "glossary_path", PXCONFIG_FMT_STRING, GlossaryPath );
	PxConfigWriteString( "main", "current_glossary", PXCONFIG_FMT_STRING, curGlos );

	for( ; i< glosSet.dataSize; i++ )
	{
		glset = GetItembyIndex( &glosSet, i );
		sprintf( buf, "\'%s\'", glset->delimiter );
		PxConfigWriteString( glset->name, "delimiter", PXCONFIG_FMT_STRING, buf );
		PxConfigWriteLong( glset->name, "skip_lines", PXCONFIG_FMT_INT_DECIMAL, glset->skip_line );
	}

	PxConfigClose();
	return 0;
}

GlossarySetting_t *getCurrentSetting( char *gl_name )
{
	int i=0;
//	GlossarySetting_t *glset;

	for( ; i< glosSet.dataSize; i++ )
	{
		if( !strcmp( (char*)GetItembyIndex( &glosSet, i ), gl_name ))
			return (GlossarySetting_t *)GetItembyIndex( &glosSet, i );
	}

	return (GlossarySetting_t *)GetItembyIndex( &glosSet, 0 );
}
