//
// library for glossary handle
//
//

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <unistd.h>
#include <mem.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <utime.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

#include "pglos.h"
#include "proto.h"

Collection *glsHash = NULL;

offset_t FindInHash( char *key )
{
	int i;
	char ch = 0;
	Hash_t *hsh;
	int l = strlen( key );

	assert( glsHash );
	assert( key );

	if( strlen( key ) > HashLenght )
	{
		ch = key[HashLenght];
		key[HashLenght] = '\0';
	}
	for( i=0; i<glsHash->dataSize; i++ )
	{
		hsh = GetItembyIndex( glsHash, i );
//		l = strlen( hsh->key ) < l ? strlen( hsh->key ) : l;
		if( *hsh->key != *key )
			continue;
		if( !strncmp( hsh->key, key, l ))
			break;
	}

	if( ch )
		key[HashLenght] = ch;
	
	return i==glsHash->dataSize? -1 : hsh->off;
}

void AddInHash( char *key, offset_t off )
{
	Hash_t *hsh = calloc( sizeof( Hash_t ), 1 );

	assert( glsHash );

	hsh->key = strdup( key );
	hsh->off = off;

	AddItem( glsHash, hsh );
}

void DestroyHash()
{
	int i=0;

	for( ; i<glsHash->dataSize; i++ )
		free( ((Hash_t*)GetItembyIndex( glsHash, i ))->key );

	RemoveAllItems( glsHash );
}

void DeleteHash()
{
	if( glsHash )
	{ 
		free( glsHash ); 
		glsHash = NULL; 
	}
}

void CreateHash( FILE *fp, char *name )
{
	char str[BUF_SIZE];
	char *ptr, *pend;
	char cur_hash[HashLenght+1] = {0};
	offset_t off;
	size_t hashsize;
	GlossarySetting_t *glset = (GlossarySetting_t *)getCurrentSetting( curGlos );

	assert( fp );

	if( !glsHash )
		glsHash = calloc( sizeof( Collection ), 1 );
	else
		DestroyHash();

	InitCollection( glsHash, 50, 50 );


	if( ReadHash( name ))
		return;

    fseek( fp, 0L, SEEK_END );
	RunHashDlg( "Create", ftell( fp ));

    fseek( fp, 0L, SEEK_SET );

	if( glset->skip_line )
	{
		int i=0;
		for(; i<glset->skip_line; i++ )
			fgets( str, sizeof(str), fp );
	}

	off = ftell( fp );
	
	fgets( str, sizeof( str ), fp );
	CHOMP( str );
	hashsize = atol( str );

	while( fgets( str, sizeof(str), fp ))
	{

		ptr = str;
		SKIP_SPACE( ptr );
		pend = strstr( ptr, glset->delimiter );
		if( pend ) *pend = '\0';
		strncpy( cur_hash, ptr, HashLenght );
		UpdateHashDlg( off );
		if( FindInHash( cur_hash ) == -1 )
			AddInHash( cur_hash, off );

		off = ftell( fp );
	}
	off = ftell( fp );
	UpdateHashDlg( off );
	DestroyHashDlg();
}

static BOOL CheckHashGlos( const char *gl_name, const char *hs_name )
{
	struct stat gbuf, hbuf;

    if( stat( gl_name, &gbuf ) == -1 ) 
		return FALSE;
    if( stat( hs_name, &hbuf ) == -1 ) 
		return FALSE;

	if( gbuf.st_mtime != hbuf.st_mtime || gbuf.st_atime != hbuf.st_atime )
		return FALSE;

	return TRUE;
}

BOOL WriteHash( const char *gl_name )
{
	char hash_name[_MAX_PATH], glos_name[_MAX_PATH];
	Hash_t *hsh;
	FILE *fp;
	int i=0;
	struct utimbuf utb;
    struct stat gbuf;

	sprintf( glos_name, "%s\%s.dwa", GlossaryPath, gl_name );
	strcpy( hash_name, glos_name );
	strcat( hash_name, ".hash" );

	if( CheckHashGlos( glos_name, hash_name )) return TRUE;

	if( !access( hash_name, W_OK ))
		unlink( hash_name );

	RunHashDlg( "Write", glsHash->dataSize );
	fp = fopen( hash_name, "wt" );
	if( !fp ) return FALSE;

	fprintf( fp, "%ld\n", glsHash->dataSize );

	for( ; i<glsHash->dataSize; i++ )
	{
		hsh = GetItembyIndex( glsHash, i );
		fprintf( fp, "%s:%ld\n", hsh->key, hsh->off );
		UpdateHashDlg( i );
	}

	fclose( fp );

	DestroyHashDlg();

    if( stat( glos_name, &gbuf ) == -1 ) 
		return FALSE;

	utb.actime = gbuf.st_atime;
	utb.modtime = gbuf.st_mtime;
	utime( hash_name, &utb );

	return TRUE;
}

BOOL ReadHash( const char *gl_name )
{
	char hash_name[_MAX_PATH], glos_name[_MAX_PATH];
	char str[1024], *ptr;
	Hash_t *hsh;
	int cnt = 0;
	FILE *fp;

	sprintf( glos_name, "%s\%s.dwa", GlossaryPath, gl_name );
	strcpy( hash_name, glos_name );
	strcat( hash_name, ".hash" );

	if( !CheckHashGlos( glos_name, hash_name )) return FALSE;

	fp = fopen( hash_name, "rt" );
	if( !fp ) return FALSE;

    fseek( fp, 0L, SEEK_END );
	RunHashDlg( "Read", ftell( fp ));

    fseek( fp, 0L, SEEK_SET );

	while( fgets( str, sizeof( str ), fp ))
	{
		ptr = strchr( str, ':' );
		if( ptr )
		{
			*ptr = '\0';
			ptr ++;

			hsh = calloc( sizeof( Hash_t ), 1 );
			hsh->key = calloc( sizeof( char ), HashLenght );
			strcpy( hsh->key, str );
			CHOMP( ptr );
			hsh->off = atol( ptr );
			AddItem( glsHash, hsh );
			cnt ++;
		}
		UpdateHashDlg( ftell( fp ));
	}

	UpdateHashDlg( ftell( fp ));
	fclose( fp );
	DestroyHashDlg();

	return cnt == glsHash->dataSize;
}
