/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.14C */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <ctype.h>
#include <assert.h>
#include <fnmatch.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "pglos.h"
#include "proto.h"

static FILE *gfp = {0};
offset_t cur_offs = 0;
#define ListSize 16

static void FillGlosList( FILE *fp, offset_t offs, short nl, char *new_key )
{
	char **lines;
	char str[BUF_SIZE*2];
	char *ptr, *pend, *ptrn;
	int i;
	GlossarySetting_t *glset = (GlossarySetting_t *)getCurrentSetting( curGlos );

	assert( fp );

	lines = (char**)calloc( sizeof( void* ), nl );

    fseek( fp, offs, SEEK_SET );
	if( !offs && glset->skip_line )
	{
		for( i=0; i< glset->skip_line; i++ )
			fgets( str, sizeof( str ), fp );
		cur_offs = ftell( fp );
	}else
	{
		cur_offs = offs;
	}

	if( new_key )
	{
		offset_t new_offs;
		do
		{
			new_offs = ftell( fp );
			fgets( str, sizeof( str ), fp );
			if( !strncmp( str, new_key, strlen( new_key )))
			{
				cur_offs = new_offs;
				fseek( fp, new_offs, SEEK_SET );
				break;
			}
			if( feof( fp ))
			{
				PtMessageBox( ABW_base, "Warning!", "The Word isn\'t Found", NULL, NULL );
				return ;
			}
		}while( 1 );
	}

	for( i=0; i< nl; i++ )
	{
		fgets( str, sizeof( str ), fp );
		ptr = str;
		SKIP_SPACE( ptr );
		pend = strstr( ptr, glset->delimiter );
		if( pend )
		{
			*pend = '\0';
			SKIP_ENDSPACE( ptr );
			ptrn = TranslateString( ptr );
			if( ptrn )
			{
				lines[i] = strdup( ptrn );
				free( ptrn );
			}else
				lines[i] = strdup( ptr );
		}
	}

	PtListDeleteAllItems( ABW_item_lst );
	PtListAddItems( ABW_item_lst, lines, i, 1 );

	for( i=0; i< nl; i++ )
		free( lines[i] );
	free( lines );
}

void LoadGlossary( char *gl_name )
{
	char fname[1024];

	sprintf( fname, "%s%s.dwa", GlossaryPath, gl_name );

	if( gfp )
		fclose( gfp );

	gfp = fopen( fname, "rt" );
	if( !gfp )
	{
		PtMessageBox( ABW_base, "Error!", "Cannot open glossary", NULL, NULL );
		return;
	}

	CreateHash( gfp, gl_name );
	FillGlosList( gfp, (offset_t)0, ListSize, NULL );
	PtListSelectPos( ABW_item_lst, 1 );
	PtListUnselectPos( ABW_item_lst, 1 );

}

int
GlosSelect( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	PtListCallback_t *cb = (PtListCallback_t*) cbinfo->cbdata;
	PtArg_t arg;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	WriteHash( curGlos );
	strcpy( curGlos, cb->item );
	LoadGlossary( cb->item );
	PtSetArg( &arg, Pt_ARG_CBOX_SEL_ITEM, cb->item_pos, 0);
	PtSetResources( ABW_gls_cmb, 1, &arg );

	return( Pt_CONTINUE );

	}

int
InitBase( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	ReadConfig();
	InitTranslation();
	SetTranslation( "Cyrillic (KOI8-R)" );

	LoadGlossary( curGlos );

	return( Pt_CONTINUE );

	}

/* Application Options string */
const char ApOptions[] =
	AB_OPTIONS ""; /* Add your options in the "" */

int
InitializeMain( int argc, char *argv[] )
{
	int i;
	
	/* eliminate 'unreferenced' warnings */
	argc = argc, argv = argv;

	for( i=1; i<argc; i++ )
	{
		if( argv[i][0] == '-' )
		{
			if(strcmp( &argv[i][1], "path" ))
				strcpy( GlossaryPath, 
					(strlen( argv[i] )==5? argv[i+1] : &argv[i][5] ));
		}
	}


	return( Pt_CONTINUE );
}

static int SearchInGlossary( FILE *fp, offset_t offs, const char *key )
{
	char str[BUF_SIZE];
	char str2[BUF_SIZE] = { 0 };
	char *ptr, *ptr2;
	GlossarySetting_t *glset = getCurrentSetting( curGlos );

	fseek( fp, offs, SEEK_SET );
	do
	{
		fgets( str, sizeof( str ), fp );
		*str2 = '\0';
		while(str[strlen(str)-1] != '\n' )
			fgets( str2, sizeof( str2 ), fp );

		ptr = str;
		SKIP_SPACE( ptr );
		ptr2 = strstr( ptr, glset->delimiter );
		if( ptr2 ) *ptr2 = '\0';
/*		pkey = key;
		while( !isalpha( *pkey ) && *pkey ) pkey ++;
		if( *ptr != *pkey )
		{
			PtMessageBox( ABW_base, "Error!", "Not found description!", NULL, NULL );
			return 0;			
		}
*/
	}while( strncmp( ptr, key, strlen( key )));
	
	SetWidgetText( ABW_srch_txt, key );

	ptr = strchr( ++ptr2, 22 );
	if( ptr )
	{
		char *items[10];
		int i=0;
		char *pend, *pbeg;
		PtArg_t arg[2];

		*ptr ='\0';
		pbeg = ++ptr;
		while( (pend = strchr( pbeg, 22 )) && i < 10 )
		{
			pend = '\0';
			SKIP_SPACE( pbeg );
			pend = strchr( pbeg , ';' );
			if( pend ) *pend = '\0';
			SKIP_ENDSPACE( pbeg );
			items[i++] = strdup( pbeg );
			pbeg = ++pend;
			SKIP_SPACE( pbeg );
			if( *pbeg == 22 ) pbeg++;
		}
		SKIP_SPACE( pbeg );
		pend = strchr( pbeg , ';' );
		if( pend ) *pend = '\0';
		pend = strchr( pbeg , '\n' );
		if( pend ) *pend = '\0';
		SKIP_ENDSPACE( pbeg );
		items[i++] = strdup( pbeg );

		EnableWidget( ABW_add_wrd_cmb );
		PtSetArg( &arg[0], Pt_ARG_ITEMS, items, i );
		PtSetArg( &arg[1], Pt_ARG_CBOX_SEL_ITEM, 1, 0);
		PtSetResources( ABW_add_wrd_cmb, 2, arg);

		while( i>0 )
			free( items[--i] );
		
	}
	ptr2 = TranslateString( ptr2 );
	if( ptr2 )
	{
		SetWidgetText( ABW_res_mtxt, ptr2 );
		free( ptr2 );
	}else
		SetWidgetText( ABW_res_mtxt, ptr );
	

	if( strlen( str2 ))
	{
		ptr2 = TranslateString( str2 );
		if( ptr2 )
		{
			AddWidgetText( ABW_res_mtxt, ptr2 );
			free( ptr2 );
		}else
			AddWidgetText( ABW_res_mtxt, str2 );
	}

	return 1;
}

int
gls_item_select( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	PtListCallback_t *cb = (PtListCallback_t *)cbinfo->cbdata;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	SearchInGlossary( gfp, cur_offs, cb->item );

	return( Pt_CONTINUE );
}


static short SearchInList( char *key )
{
	PtArg_t arg;
	short *num, i;
	char **items = NULL;
	static short sel = -1;

	PtSetArg( &arg, Pt_ARG_ITEMS, &items, &num );
	PtGetResources(ABW_item_lst, 1, &arg );

	if( sel != -1 )
		PtListUnselectPos( ABW_item_lst, sel );

	for( i = 0; i < *num; i++)
	{	
		if( !strncmp( items[i], key, strlen ( key ) ))
		{
			sel = i+1;
			break;
		}
	}

	PtListSelectPos( ABW_item_lst, sel );
	return i == *num ? -1 : i ;
}

int
input_mask( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	PtTextCallback_t *cb = (PtTextCallback_t *)cbinfo->cbdata;
	offset_t offs;
	PtArg_t arg;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	DisableWidget( ABW_add_wrd_cmb );
	PtSetArg( &arg, Pt_ARG_ITEMS, NULL, 0 );
	PtSetResources( ABW_add_wrd_cmb, 1, &arg);

	while( SearchInList( cb->text ) == -1 )
	{
		offs = FindInHash( cb->text );
		if( offs == -1 )
		{
			PtMessageBox( ABW_base, "Not Found!", cb->text, NULL, NULL );
			return( Pt_CONTINUE );
		} else
		{
			FillGlosList( gfp, offs, ListSize, 
				strlen( cb->text ) > HashLenght ? cb->text : NULL );
		}
	}

	return( Pt_CONTINUE );
}


int
ShutDown( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	if( gfp ) fclose( gfp );
	WriteHash( curGlos );
	DestroyHash();
	DoneTranslation();

	WriteConfig();

	return( Pt_CONTINUE );

}

int
add_wrd_sel( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	PtListCallback_t *cb = (PtListCallback_t*) cbinfo->cbdata;
	offset_t offs;
	char *ptr;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	ptr = UnTranslateString( cb->item );
	offs = FindInHash( ptr?ptr:cb->item );

	if( offs == -1 )
	{
		PtMessageBox( ABW_base, "Not Found!", cb->item, NULL, NULL );
	} else
	{
		SearchInGlossary( gfp, offs, ptr?ptr:cb->item );
		FillGlosList( gfp, offs, ListSize, NULL );
		SearchInList( cb->item );
	}

	if( ptr ) free( ptr );
	
	return( Pt_CONTINUE );
}


int
srch_word_key( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	PhEvent_t *event = cbinfo ? cbinfo->event : NULL;
	PhPointerEvent_t *ptr = 
		event ? (PhPointerEvent_t *)PhGetData( event ) : NULL;
	PtArg_t arg[2];
	short *num;
	char **items = NULL;
	unsigned short *selcnt, *sel;


	/* post the popup if the right button was pressed */
	if( event && (event->type & Ph_EV_KEY) && (ptr->key_mods == 150 )) // kb_enter
	{
		PtSetArg( &arg[0], Pt_ARG_ITEMS, &items, &num );
		PtSetArg( &arg[1], Pt_ARG_SELECTION_INDEXES, &selcnt, &sel );
		PtGetResources( ABW_item_lst, 2, arg );
		if( selcnt )
			SearchInGlossary( gfp, cur_offs, items[*selcnt-1]);
	}

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}
