/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

#include <stdlib.h>
#include <string.h>

#include "translate.h"
#include "abimport.h"

char *translocal[64];
char *transdesc[64];
static int TranslateType=0;
int TransCnt = 0;
struct PxTransCtrl *trans = NULL;

void DoneTranslation()
{
	int i;

	for( i=0; i<TransCnt; i++ )
	{
		free(transdesc[i]);
		free(translocal[i]);
	}
}

void InitTranslation(void)
{
	char *section, *cp, tmp[_MAX_PATH + 1];
	int maxsz= sizeof(translocal) / sizeof(translocal[0]);

	if ((cp= getenv("PHOTON_PATH")) == NULL)
		cp= "/usr/photon";
	else if (!strcmp(cp, "/"))
		cp= "";
	sprintf(tmp, "%s/translations/charsets", cp);
	if (!PxConfigOpen(tmp, PXCONFIG_READ))
	{
		PtAskQuestion(NULL, "Error", "Can't open charsets file!",
			"helv12", " &Close ", NULL, NULL, 1 );
		return;
	}

	TransCnt = 0;
	while ((section= PxConfigNextSection()) != NULL)
	{

		if (TransCnt == maxsz)
		{
			sprintf(tmp, "%s/translations/charsets", cp);
			break;
		}


		if (PxConfigReadString(NULL, "Description", "", tmp, sizeof(tmp)))
		{
			if( !strcmp( section, "KOI8-R"))
				TranslateType = TransCnt;

			transdesc[TransCnt] = strdup( tmp );
			if (PxConfigReadString(NULL, "MIME", "", tmp, sizeof(tmp)))
				translocal[TransCnt] = strdup( tmp);
			else
			{
				strcpy(tmp, transdesc[TransCnt]);
				if ((cp= strtok(tmp, " \t\n")) != NULL)
					translocal[TransCnt] = strdup( cp );
				else
					strcpy( translocal[TransCnt], transdesc[TransCnt]);
			}
			TransCnt++;
		}

	}			// while

	PxConfigClose();
	SetTranslateType( TranslateType );
	
}				// init_read_translations


int GetTranslateType()
{
	return TranslateType;
}

int SetTranslation( const char *tr )
{
	int i=0;

	for( ; i< TransCnt; i++ )
		if( !strcmp( tr, transdesc[i] ))
			break;

	return SetTranslateType( i );
}

int SetTranslateType( int tr )
{
//	if( tr == TranslateType )
//		return 0;

	if( tr < 0 || tr >= TransCnt )
	{
		PtMessageBox( ABW_base, "Error!", "Error Translation Number!", NULL, NULL );
		return -1;
	}
	
	trans = PxTranslateSet( NULL, translocal[TranslateType=tr] );
	if( trans == NULL )
	{
		char err_str[256];
		
		TranslateType = -1;
		sprintf( err_str, "unknow translation charset-%s\n", 
			translocal[tr]);
		PtMessageBox( ABW_base, "Error!", err_str, NULL, NULL );

		return -1;
	}

	return 0;
}

char *TranslateString(char *src )
{
	int srctaken, dstmade;
	char *dest = NULL;

	dstmade = dstmade;
	if( src )
	{
		dstmade = PxTranslateStateToUTF( trans, src, strlen( src ), &srctaken,
			NULL, dstmade );
			
		if( dstmade >0 )
		{
			dest = malloc(dstmade + 1);
			PxTranslateStateToUTF( trans, src, strlen( src ), &srctaken,
				dest, dstmade );
			dest[dstmade] = '\0';	
			
		}
	}
	
	return dest;
}

char *UnTranslateString(char *src )
{
	int srctaken, dstmade;
	char *dest = NULL;

	dstmade = dstmade;

	if( src )
	{
		PxTranslateFromUTF( trans, src, strlen( src ), &srctaken,
			NULL, dstmade, &dstmade );
			
		if( dstmade >0 )
		{
			dest = malloc(dstmade + 1);
			PxTranslateFromUTF( trans, src, strlen( src ), &srctaken,
				dest, dstmade, &dstmade );
			dest[dstmade] = '\0';	
		}
	}
	
	return dest;
}
